{% extends "base.html" %}

{% block title %}404 Not Found{% endblock %}
{% block content %}
  <h1>Not Found</h1>
  <img src="{{ url_for('static', filename='giorgio.jpg') }}" alt=""/>
  <p>The requested URL was not found on the server. If you entered the URL manually please check your spelling and try again.</p>
{% endblock %}
